"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const minikin_1 = require("minikin");
const fs = require("fs");
const __1 = require("../..");
const mustache = require("mustache");
const getPath = (fileName) => {
    return fs.realpathSync(`${__dirname}/../web/${fileName}`);
};
const fromFile = (fileName) => minikin_1.Response.fromFile(getPath(fileName));
const getTemplate = (fileName) => {
    try {
        const filePath = getPath(fileName);
        return fs.readFileSync(filePath, "utf8");
    }
    catch (ex) {
        console.log(ex);
        return "";
    }
};
const render = (fileName, data) => {
    return minikin_1.Response.fromString(mustache.render(getTemplate(fileName), data)).header("Content-Type", "text/html");
};
const routes = {
    "GET /": () => render("index.html", {
        suites: Object.values(__1.FlagpoleExecution.global.config.suites),
    }),
    "GET /style.css": () => fromFile("style.css"),
};
class Serve extends command_1.Command {
    constructor() {
        super(...arguments);
        this.commandString = "serve";
        this.description = "Run a local web server to interact with Flagpole";
    }
    action() {
        return __awaiter(this, void 0, void 0, function* () {
            const server = yield minikin_1.default.server(8000);
            server.routes(routes);
        });
    }
}
exports.default = Serve;
//# sourceMappingURL=serve.js.map