import { iConsoleLine, LineType } from "../interfaces/iconsole-log";
export declare abstract class ConsoleLine implements iConsoleLine {
    timestamp: Date;
    textPrefix: string;
    textSuffix: string;
    message: string;
    fg: [number, number, number];
    type: LineType;
    static targetLineLength: number;
    constructor(message: string);
    toString(): string;
    toConsoleString(): string;
}
export declare class HeadingLine extends ConsoleLine implements iConsoleLine {
    constructor(message: string);
    toString(): string;
}
export declare class SubheadingLine extends ConsoleLine implements iConsoleLine {
    constructor(message: string);
    toString(): string;
}
export declare class SectionHeadingLine extends ConsoleLine implements iConsoleLine {
    constructor(message: string);
    toString(): string;
}
export declare class CustomLine extends ConsoleLine implements iConsoleLine {
    constructor(message: string, fg: [number, number, number]);
}
export declare class LineBreak extends ConsoleLine implements iConsoleLine {
    constructor();
}
export declare class CommentLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class PassLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class ActionCompletedLine extends PassLine implements iConsoleLine {
    protected _verb: string;
    protected _noun: string;
    constructor(verb: string, noun: string);
    toConsoleString(): string;
}
export declare class FailLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class ActionFailedLine extends FailLine implements iConsoleLine {
    protected _verb: string;
    protected _noun: string;
    constructor(verb: string, noun: string);
    toConsoleString(): string;
}
export declare class OptionalFailLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class WarningLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class DetailLine extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    constructor(message: string);
    toConsoleString(): string;
}
export declare class ErrorActualValueLine extends DetailLine {
    constructor(message: string);
}
export declare class SourceCodeBlock extends ConsoleLine implements iConsoleLine {
    textPrefix: string;
    protected highlight: string | null;
    constructor(message: string, highlight?: string);
    toConsoleString(): string;
    protected _codeHighlight(source: string): string;
}
export declare class ErrorSourceCodeBlock extends SourceCodeBlock {
    constructor(message: string, highlight?: string);
}
