"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HlsResponse = void 0;
const parse_hls_1 = require("parse-hls");
const helpers_1 = require("../helpers");
const jpath_1 = require("../json/jpath");
const media_response_1 = require("./media-response");
class HlsResponse extends media_response_1.MediaResponse {
    constructor() {
        super(...arguments);
        this._mimePattern = /mpegurl|octet-stream/i;
        this.find = (path) => jpath_1.jpathFind(this, path);
        this.findAll = (path) => jpath_1.jpathFindAll(this, path);
    }
    get jsonBody() {
        var _a;
        return helpers_1.wrapAsValue(this.context, (_a = this.jsonDoc) === null || _a === void 0 ? void 0 : _a.root, "Parsed Manifest");
    }
    get isM3U8() {
        return this.body.toString().trim().startsWith("#EXTM3U");
    }
    init(httpResponse) {
        super.init(httpResponse);
        this.context
            .assert("Content looks like M3U format", this.isM3U8)
            .equals(true);
        try {
            const manifest = parse_hls_1.default.parse(httpResponse.body);
            this.jsonDoc = new jpath_1.JsonDoc(manifest.serialize());
        }
        catch (ex) {
            this.context.logFailure("Error parsing HLS manifest.", ex);
        }
    }
}
exports.HlsResponse = HlsResponse;
//# sourceMappingURL=hls-response.js.map