"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AtomResponse = void 0;
const xml_response_1 = require("./xml-response");
const validMimeTypes = ["application/atom+xml", "text/xml", "text/atom+xml"];
class AtomResponse extends xml_response_1.XmlResponse {
    init(httpResponse) {
        var _a;
        super.init(httpResponse);
        this.context.assert(this.statusCode).between(200, 299);
        const mimeType = (_a = String(this.header("Content-Type").$)
            .split(";")
            .shift()) === null || _a === void 0 ? void 0 : _a.trim();
        this.context
            .assert(`Mime Type is valid for Atom (${validMimeTypes.join(", ")})`, mimeType)
            .in(validMimeTypes);
        this.context
            .assert("Has required Atom fields", this.hasRequiredAtomFields())
            .equals(true);
    }
    hasRequiredAtomFields() {
        const feed = this.cheerio("feed");
        if (feed.length !== 1) {
            return false;
        }
        const channelId = feed.children("id");
        const channelTitle = feed.children("title");
        const channelUpdated = feed.children("updated");
        if (channelId.length !== 1 ||
            channelTitle.length != 1 ||
            channelUpdated.length != 1) {
            return false;
        }
        const entries = feed.children("entry");
        if (entries.length > 0) {
            let allItemsAreValid = true;
            entries.each((i, entry) => {
                const id = this.cheerio(entry).children("id");
                const title = this.cheerio(entry).children("title");
                const updated = this.cheerio(entry).children("updated");
                if (id.length !== 1 || title.length !== 1 || updated.length !== 1) {
                    allItemsAreValid = false;
                    return false;
                }
            });
            if (!allItemsAreValid) {
                return false;
            }
        }
        return true;
    }
}
exports.AtomResponse = AtomResponse;
//# sourceMappingURL=atom-response.js.map